﻿using System;
using System.Collections.Generic;
using System.Text;
    
using System.Data;
using System.Text.RegularExpressions;
    
namespace CreateDataSet
{
    public enum SelectTicket
    {
        AB,
        CD,
        ABCD
    }
    
    class CreateData
    {
        // Source: "http://www.dotnetfunda.com/articles/article51.aspx"
        void ConvertHTMLTablesToDataSet(
            String HTML, DataSet dsQuestions, DataSet dsAnswers)
        {
            // Локальные сылки
            DataTable dtQuestions;
            DataRow drQuestions;
            DataTable dtAnswers;
            DataRow drAnswers;
    
            bool HeadersExist = false;  // Флаг существования заголовка таблицы
            int iCurrentColumn = 0;     // Счетчик столбцов
            int iCurrentRow = 0;        // Счетчик строк
    
            // Регулярные выражения и опции
            String TableExpression = "<table[^>]*>(.*?)</table>";
            String HeaderExpression = "<th[^>]*>(.*?)</th>";
            String RowExpression = "<tr[^>]*>(.*?)</tr>";
            String ColumnExpression = "<td[^>]*>(.*?)</td>";
            String ParagraphExpression = "<p[^>]*>(.*?)</p>";
            RegexOptions options = RegexOptions.Multiline |
                                   RegexOptions.Singleline |
                                   RegexOptions.IgnoreCase;
    
            // Извлекаем коллекцию таблиц из содержимого HTML
            MatchCollection Tables = Regex.Matches(HTML, TableExpression, options);
    
            // Проходим по всем HTML-таблицам 
            foreach (Match Table in Tables)
            {
                // Сбрасываем счетчик строки и флаг заголовка
                HeadersExist = false;
                iCurrentRow = 0;
    
                // Создаем объект новой таблицы и наполняем его
                dtQuestions = new DataTable();
                dtAnswers = new DataTable();
    
                /* Создаем соответствующее количество столбцов 
                 * для текущей таблицы (используем заголовки, 
                 * если они существуют, иначе используем 
                 * заданные по умолчанию названия)                
                 */
                if (Table.Value.Contains("<th"))
                {
                    HeadersExist = true;// Таблица содержит заголовок
                    // Заполняем коллекцию соответствий для всех <th> в таблице
                    MatchCollection Headers = Regex.Matches(
                        Table.Value, HeaderExpression, options);
    
                    // Добавляем столбцы с именами из строки заголовков текущей таблицы
                    foreach (Match Header in Headers)
                    {
                        // Groups[0] - с дескрипторами <p>Содержимое</p>
                        // Groups[1] - только Содержимое
                        MatchCollection paragraphs = Regex.Matches(Header.Value, ParagraphExpression, options);
                        if (paragraphs.Count != 0)
                            dtQuestions.Columns.Add(paragraphs[0].Groups[1].ToString());
                        else
                            dtQuestions.Columns.Add(Header.Groups[1].ToString());
                    }
                }
                else
                {
                    Match tab = Regex.Matches(Table.Value, TableExpression, options)[0];
                    Match row = Regex.Matches(tab.ToString(), RowExpression, options)[0];
                    int count = Regex.Matches(row.ToString(), ColumnExpression, options).Count;
    
                    // Строки заголовков нет, добавляем дежурные имена
                    for (int iColumns = 0; iColumns < count; iColumns++)
                        dtQuestions.Columns.Add("Column" + iColumns);
                }
    
                String[] TableAnswerColumnsName = { "QueryID", "AnswerID", "Answer" };
                for (int iColumns = 0; iColumns < TableAnswerColumnsName.Length; iColumns++)
                    dtAnswers.Columns.Add(TableAnswerColumnsName[iColumns]);
    
                // Извлекаем коллекцию строк текущей таблицы
                MatchCollection Rows = Regex.Matches(Table.Value, RowExpression, options);
    
                // Добавляем строки в объект DataRow
                foreach (Match Row in Rows)
                {
                    // Игнорируем строку заголовка
                    if (iCurrentRow != 0 || HeadersExist != true)
                    {
                        drQuestions = dtQuestions.NewRow();
                        iCurrentColumn = 0;
                        // Извлекаем коллекцию столбцов текущей строки
                        MatchCollection Columns = Regex.Matches(Row.Value, ColumnExpression, options);
                        foreach (Match Column in Columns)
                        {
                            // Извлекаем коллекцию параграфов из текущего столбца
                            MatchCollection paragraphs = Regex.Matches(Column.Value, ParagraphExpression, options);
                            if (paragraphs.Count != 0)
                            {
                                int iAnswer = 0;
                                String line = String.Empty;
                                foreach (Match paragraph in paragraphs)
                                {
                                    line = paragraph.Groups[1].ToString();
    
                                    // Убираем все лишнее
                                    string[] split = line.Split(new Char[] { '\r', '\n', ' ' });
                                    StringBuilder strBuilder = new StringBuilder();
                                    foreach (string s in split)
                                    {
                                        string word = s.Trim();
                                        if (word != String.Empty)
                                            strBuilder.Append(" " + word);
                                    }
                                    line = strBuilder.ToString().Trim();
    
                                    // Убираем номера ответов
                                    int pos = line.IndexOf(".");
                                    if (pos > 0 && pos <= 2 && iCurrentColumn == 1)
                                    {
                                        // Ответы 
                                        drAnswers = dtAnswers.NewRow();
                                        drAnswers[0] = drQuestions[0];
                                        iAnswer++;
                                        drAnswers[1] = iAnswer.ToString();
                                        line = line.Substring(pos + 1).Trim();
                                        drAnswers[2] = line;
                                        dtAnswers.Rows.Add(drAnswers);
                                        continue;
                                    }
    
                                    if (iAnswer == 0)
                                        drQuestions[iCurrentColumn] = line;
                                }
                            }
                            else
                                drQuestions[iCurrentColumn] = Column.Groups[1].ToString();
    
                            iCurrentColumn++;
                        }
    
                        dtQuestions.Rows.Add(drQuestions);
                    }
    
                    iCurrentRow++;
                }
    
                dsQuestions.Tables.Add(dtQuestions);
                dsAnswers.Tables.Add(dtAnswers);
            }
        }
    
        void FillDataset(string path, DataSet dsQuestions, DataSet dsAnswers, int count)
        {
            for (int i = 1; i <= count; i++)
            {
                string file = path + "\\Ticket" + i.ToString() + ".htm";
                System.IO.StreamReader streamReader = new System.IO.StreamReader(
                    file, Encoding.GetEncoding(1251));
    
                String source = streamReader.ReadToEnd();
    
                DataSet dsQuestionsItem = new DataSet();
                DataSet dsAnswersItem = new DataSet();
                ConvertHTMLTablesToDataSet(source, dsQuestionsItem, dsAnswersItem);
    
                foreach (DataTable table in dsQuestionsItem.Tables)
                {
                    DataRow[] rows = new DataRow[table.Rows.Count];
                    for (int iRow = 0; iRow < rows.Length; iRow++)
                    {
                        rows[iRow] = table.Rows[iRow];
                        rows[iRow][0] = (rows[iRow][0]).ToString().Trim();
                    }
                    dsQuestions.Merge(rows);
                }
                foreach (DataTable table in dsAnswersItem.Tables)
                {
                    DataRow[] rows = new DataRow[table.Rows.Count];
                    for (int iRow = 0; iRow < rows.Length; iRow++)
                    {
                        rows[iRow] = table.Rows[iRow];
                        rows[iRow][0] = (rows[iRow][0]).ToString().Trim();
                    }
                    dsAnswers.Merge(rows);
                }
            }
        }
    
        // Поля
        DataSet dsQuestionsAB = new DataSet();
        DataSet dsAnswersAB = new DataSet();
        DataSet dsQuestionsCD = new DataSet();
        DataSet dsAnswersCD = new DataSet();
        int lenQuery = 0, lenComment = 0, lenAnswer = 0;
        int countTicket = 40; // Количество файлов-билетов
    
        // Свойства
        public DataSet QuestionsAB { get { return dsQuestionsAB; } }
        public DataSet AnswersAB { get { return dsAnswersAB; } }
        public DataSet QuestionsCD { get { return dsQuestionsCD; } }
        public DataSet AnswersCD { get { return dsAnswersCD; } }
        public int LenQuery { get { return lenQuery; } }
        public int LenComment { get { return lenComment; } }
        public int LenAnswer { get { return lenAnswer; } }
        public int CountTicket { set { countTicket = value; } }
    
        public void Execute(SelectTicket selectTicket)
        {
            string path = System.IO.Directory.GetCurrentDirectory();
    
            int len;
            switch (selectTicket)
            {
                case SelectTicket.AB:
                    // Заполняем DataSet
                    FillDataset(path + @"\AB", dsQuestionsAB, dsAnswersAB, countTicket);
    
                    // Считаем размеры полей для создания объектных таблиц
                    foreach (DataRow row in dsQuestionsAB.Tables[0].Rows)
                    {
                        len = row["Query"].ToString().Length;
                        if (len > lenQuery)
                            lenQuery = len;
    
                        len = row["Comment"].ToString().Length;
                        if (len > lenComment)
                            lenComment = len;
                    }
                    foreach (DataRow row in dsAnswersAB.Tables[0].Rows)
                    {
                        len = row["Answer"].ToString().Length;
                        if (len > lenAnswer)
                            lenAnswer = len;
                    }
                    break;
                case SelectTicket.CD:
                    // Заполняем DataSet
                    FillDataset(path + @"\CD", dsQuestionsCD, dsAnswersCD, countTicket);
    
                    // Считаем размеры полей для создания объектных таблиц
                    foreach (DataRow row in dsQuestionsCD.Tables[0].Rows)
                    {
                        len = row["Query"].ToString().Length;
                        if (len > lenQuery)
                            lenQuery = len;
    
                        len = row["Comment"].ToString().Length;
                        if (len > lenComment)
                            lenComment = len;
                    }
                    foreach (DataRow row in dsAnswersCD.Tables[0].Rows)
                    {
                        len = row["Answer"].ToString().Length;
                        if (len > lenAnswer)
                            lenAnswer = len;
                    }
                    break;
                case SelectTicket.ABCD:
                    // Заполняем DataSet
                    FillDataset(path + @"\AB", dsQuestionsAB, dsAnswersAB, countTicket);
                    FillDataset(path + @"\CD", dsQuestionsCD, dsAnswersCD, countTicket);
    
                    // Считаем размеры полей для создания объектных таблиц
                    foreach (DataRow row in dsQuestionsAB.Tables[0].Rows)
                    {
                        len = row["Query"].ToString().Length;
                        if (len > lenQuery)
                            lenQuery = len;
    
                        len = row["Comment"].ToString().Length;
                        if (len > lenComment)
                            lenComment = len;
                    }
                    foreach (DataRow row in dsAnswersAB.Tables[0].Rows)
                    {
                        len = row["Answer"].ToString().Length;
                        if (len > lenAnswer)
                            lenAnswer = len;
                    }
                    foreach (DataRow row in dsQuestionsCD.Tables[0].Rows)
                    {
                        len = row["Query"].ToString().Length;
                        if (len > lenQuery)
                            lenQuery = len;
    
                        len = row["Comment"].ToString().Length;
                        if (len > lenComment)
                            lenComment = len;
                    }
                    foreach (DataRow row in dsAnswersCD.Tables[0].Rows)
                    {
                        len = row["Answer"].ToString().Length;
                        if (len > lenAnswer)
                            lenAnswer = len;
                    }
                    break;
            }
        }
    
        public void Show(DataSet dsQuestions, DataSet dsAnswers)
        {
            for (int i = 0; i < dsQuestions.Tables.Count; i++)
            {
                // Распечатываем заголовки столбцов
                foreach (DataColumn col in dsQuestions.Tables[i].Columns)
                    Console.WriteLine("{0}", col.ColumnName);
                Console.WriteLine();
    
                // Распечатываем строки
                foreach (DataRow row in dsQuestions.Tables[i].Rows)
                {
                    foreach (DataColumn col in dsQuestions.Tables[i].Columns)
                        Console.WriteLine("{0}", row[col].ToString());
                    Console.WriteLine();
                }
    
                Console.WriteLine();
            }
    
            for (int i = 0; i < dsAnswers.Tables.Count; i++)
            {
                // Распечатываем заголовки столбцов
                foreach (DataColumn col in dsAnswers.Tables[i].Columns)
                    Console.WriteLine("{0}", col.ColumnName);
                Console.WriteLine();
    
                // Распечатываем строки
                foreach (DataRow row in dsAnswers.Tables[i].Rows)
                {
                    foreach (DataColumn col in dsAnswers.Tables[i].Columns)
                        Console.WriteLine("{0}", row[col].ToString());
                    Console.WriteLine();
                }
    
                Console.WriteLine();
            }
        }
    }
}