﻿
using System;
using System.Windows;           // DependencyObject, DependencyProperty
using System.ComponentModel;    // INotifyPropertyChanged
using System.Windows.Threading; // DispatcherTimer
using System.Globalization;     // CultureInfo 
using System.Windows.Controls;  // TextBlock
using System.Windows.Media;     // FontFamily

namespace MyClock
{
    public class DigitalClock : TextBlock, INotifyPropertyChanged
    {
        const int COUNT_TIME = 10;    // Минуты

        // Событие реализации интерфейса INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        // Обычное событие остановки таймера
        public event EventHandler StopDigitalClock;

        // Открытое свойство 
        int minutes, seconds, time;
        public TextBlock Time
        {
            get
            {
                minutes = time / 60;
                seconds = time % 60;
                this.Text = String.Format("{0:00}:{1:D2}", minutes, seconds);
                if (time > 0)
                    time -= 1;
                return this;
            }
        }

        // Конструктор запускает таймер
        DispatcherTimer timer;
        public DigitalClock()
        {
            // Настраиваем дизайн
            this.FontFamily = new FontFamily("Arial");
            this.FontSize = 1.3333 * 36;    // 36pt
            this.FontWeight = FontWeights.Bold;
            this.Background = Brushes.Red;
            this.Foreground = Brushes.Yellow;
            this.Padding = new Thickness(5, 0, 5, 0);

            // Создаем таймер
            timer = new DispatcherTimer();
            timer.Tick += new EventHandler(timer_Tick);
            timer.Interval = TimeSpan.FromSeconds(1);

            // Начальное значение
            time = COUNT_TIME * 60;   // Секунды
        }

        public void Start()
        {
            timer.Stop();
            timer.Start();
        }

        public void Restart()
        {
            time = COUNT_TIME * 60;   // Секунды
            timer.Stop();
            timer.Start();
        }

        public void StopResult()
        {
            timer.Stop();
            if (StopDigitalClock != null)
                StopDigitalClock(this, EventArgs.Empty);
        }

        public void Stop()
        {
            timer.Stop();
        }

        // Обработчик события таймера возбуждает свойство Time
        // и генерирует обычное событие останова 
        void timer_Tick(object sender, EventArgs e)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs("Time"));

            if (minutes <= 0 && seconds <= 0)
            {
                timer.Stop();
                if (StopDigitalClock != null)
                    StopDigitalClock(this, EventArgs.Empty);
            }
        }
    }
}